#include <stdlib.h>

#include "image.h"
#include "iterFunc.h"

#define RED 0
#define GREEN 1
#define BLUE 2

image::~image()
{
    delete[] buffer;
    delete[] iter_buf;
    free(data_buf);
}

image::image()
{
    m_Xres = m_Yres = 0;
    buffer = NULL;
    iter_buf = NULL;
    data_buf = NULL;
    data_size = 0;
}

inline int
image::row_length() const
{
    return m_Xres * 3;
}

inline int
image::bytes() const
{
    return row_length() * m_Yres;
}

void 
image::put(int x, int y, rgb_t pixel)
{
    char *start = buffer + x*3 + y * row_length();
    start[RED] = pixel.r;
    start[GREEN] = pixel.g;
    start[BLUE] = pixel.b;
}

rgb_t 
image::get(int x, int y) const
{
    char *start = buffer + x*3 + y * row_length();
    rgb_t pixel;
    pixel.r = start[RED];
    pixel.g = start[GREEN];
    pixel.b = start[BLUE];
    
    return pixel;
}

image::image(const image& im)
{
    m_Xres = im.m_Xres;
    m_Yres = im.m_Yres;
    data_size = im.data_size;
    buffer = new char[bytes()];
    iter_buf = new int[m_Xres * m_Yres];
    data_buf = malloc(data_size * m_Xres * m_Yres);
}

bool image::set_resolution(int x, int y)
{
    if(buffer && m_Xres == x && m_Yres == y) return 0;
    m_Xres = x;
    m_Yres = y;
    delete[] buffer;
    delete[] iter_buf;
    free(data_buf);
    buffer = new char[bytes()];
    iter_buf = new int[m_Xres * m_Yres];
    data_buf = malloc(data_size * m_Xres * m_Yres);
    return 1;
}

double image::ratio() const
{
    return ((double)m_Yres / m_Xres);
}

void image::clear()
{
    // no need to clear image buffer
    for(int i = 0; i < m_Xres * m_Yres; i++) {
        iter_buf[i]=-1;
    }
}

bool image::set_data_size(int size)
{
    if(size == data_size) return false;
    data_size = size;
    data_buf = realloc(data_buf,size * m_Xres * m_Yres);
    return true;
}
	       
